@@ -47,7 +47,40 @@ NUMBER _qlge_ = { { _qlgenum_, 1, 0 }, { _qlgeden_, 1, 0 }, 1, NULL };
 STATIC NUMBER *ln_10 = NULL;
 STATIC NUMBER *ln_10_epsilon = NULL;
 
-STATIC NUMBER *pivalue[2];
+/*
+ * cache pi
+ *
+ * pivalue[LAST_PI_EPSILON] - last epsilon used to calculate pi
+ * pivalue[LAST_PI_VALUE] - last calculated pi
+ *			      given pivalue[LAST_PI_EPSILON] epsilon
+ * pivalue[LAST_PI_DIV_180_EPSILON] - last epsilon used to calculate pi/180
+ * pivalue[LAST_PI_DIV_180_VALUE] - last calculated pi/180 given
+ *				      pivalue[LAST_PI_DIV_180_EPSILON] epsilon
+ * pivalue[LAST_PI_DIV_200_EPSILON] - last epsilon used to calculate pi/200
+ * pivalue[LAST_PI_DIV_200_VALUE] - last calculated pi/200 given
+ *				      pivalue[LAST_PI_DIV_200_EPSILON] epsilon
+ */
+enum pi_cache {
+	LAST_PI_EPSILON = 0,
+	LAST_PI_VALUE,
+	LAST_PI_DIV_180_EPSILON,
+	LAST_PI_DIV_180_VALUE,
+	LAST_PI_DIV_200_EPSILON,
+	LAST_PI_DIV_200_VALUE,
+	PI_CACHE_LEN	/* must be last */
+};
+STATIC NUMBER *pivalue[PI_CACHE_LEN] = {
+	NULL,	/* LAST_PI_EPSILON */
+	NULL,	/* LAST_PI_VALUE */
+	NULL,	/* LAST_PI_DIV_180_EPSILON */
+	NULL,	/* LAST_PI_DIV_180_VALUE */
+	NULL,	/* LAST_PI_DIV_200_EPSILON */
+	NULL,	/* LAST_PI_DIV_200_VALUE */
+};
+
+/*
+ * other static function decls
+ */
 STATIC NUMBER *qexprel(NUMBER *q, long bitnum);
 
 /*
@@ -725,16 +758,25 @@ qpi(NUMBER *epsilon)
 	long bits;			/* needed number of bits of precision */
 	long t;
 
+	/* firewall */
 	if (qiszero(epsilon)) {
 		math_error("zero epsilon value for pi");
 		/*NOTREACHED*/
 	}
-	if (epsilon == pivalue[0])
-		return qlink(pivalue[1]);
-	if (pivalue[0]) {
-		qfree(pivalue[0]);
-		qfree(pivalue[1]);
+
+	/* use pi cache if epsilon marches, else flush if needed */
+	if (pivalue[LAST_PI_EPSILON] != NULL &&
+	    pivalue[LAST_PI_VALUE] != NULL &&
+	    epsilon == pivalue[LAST_PI_EPSILON]) {
+		return qlink(pivalue[LAST_PI_VALUE]);
 	}
+	if (pivalue[LAST_PI_EPSILON] != NULL) {
+		qfree(pivalue[LAST_PI_EPSILON]);
+	}
+	if (pivalue[LAST_PI_VALUE] != NULL) {
+		qfree(pivalue[LAST_PI_VALUE]);
+	}
+
 	bits = -qilog2(epsilon) + 4;
 	if (bits < 4)
 		bits = 4;
@@ -768,11 +810,100 @@ qpi(NUMBER *epsilon)
 	zfree(sum);
 	r = qmappr(t1, epsilon, 24L);
 	qfree(t1);
-	pivalue[0] = qlink(epsilon);
-	pivalue[1] = qlink(r);
+	pivalue[LAST_PI_EPSILON] = qlink(epsilon);
+	pivalue[LAST_PI_VALUE] = qlink(r);
 	return r;
 }
 
+
+/*
+ * qpidiv180 - calcucalte pi / 180
+ *
+ * This function returns pi/180 as used to covert between radians and degrees.
+ */
+NUMBER *
+qpidiv180(NUMBER *epsilon)
+{
+	NUMBER *pi, *pidiv180;
+
+	/* firewall */
+	if (qiszero(epsilon)) {
+		math_error("zero epsilon value for qpidiv180");
+		/*NOTREACHED*/
+	}
+
+	/* use pi/180 cache if epsilon marches, else flush if needed */
+	if (pivalue[LAST_PI_DIV_180_EPSILON] != NULL &&
+	    pivalue[LAST_PI_DIV_180_VALUE] != NULL &&
+	    epsilon == pivalue[LAST_PI_DIV_180_EPSILON]) {
+		return qlink(pivalue[LAST_PI_DIV_180_VALUE]);
+	}
+	if (pivalue[LAST_PI_DIV_180_EPSILON] != NULL) {
+		qfree(pivalue[LAST_PI_DIV_180_EPSILON]);
+	}
+	if (pivalue[LAST_PI_DIV_180_VALUE] != NULL) {
+		qfree(pivalue[LAST_PI_DIV_180_VALUE]);
+	}
+
+	/* let qpi() returned cached pi or calculate new as needed */
+	pi = qpi(epsilon);
+
+	/* calculate pi/180 */
+	pidiv180 = qdivi(pi, 180);
+
+	/* cache epsilon and pi/180 */
+	pivalue[LAST_PI_DIV_180_EPSILON] = qlink(epsilon);
+	pivalue[LAST_PI_DIV_180_VALUE] = qlink(pidiv180);
+
+	/* return pi/180 */
+	return pidiv180;
+}
+
+
+/*
+ * qpidiv200 - calcucalte pi / 200
+ *
+ * This function returns pi/200 as used to covert between radians and gradians.
+ */
+NUMBER *
+qpidiv200(NUMBER *epsilon)
+{
+	NUMBER *pi, *pidiv200;
+
+	/* firewall */
+	if (qiszero(epsilon)) {
+		math_error("zero epsilon value for qpidiv200");
+		/*NOTREACHED*/
+	}
+
+	/* use pi/200 cache if epsilon marches, else flush if needed */
+	if (pivalue[LAST_PI_DIV_200_EPSILON] != NULL &&
+	    pivalue[LAST_PI_DIV_200_VALUE] != NULL &&
+	    epsilon == pivalue[LAST_PI_DIV_200_EPSILON]) {
+		return qlink(pivalue[LAST_PI_DIV_200_VALUE]);
+	}
+	if (pivalue[LAST_PI_DIV_200_EPSILON] != NULL) {
+		qfree(pivalue[LAST_PI_DIV_200_EPSILON]);
+	}
+	if (pivalue[LAST_PI_DIV_200_VALUE] != NULL) {
+		qfree(pivalue[LAST_PI_DIV_200_VALUE]);
+	}
+
+	/* let qpi() returned cached pi or calculate new as needed */
+	pi = qpi(epsilon);
+
+	/* calculate pi/200 */
+	pidiv200 = qdivi(pi, 200);
+
+	/* cache epsilon and pi/200 */
+	pivalue[LAST_PI_DIV_200_EPSILON] = qlink(epsilon);
+	pivalue[LAST_PI_DIV_200_VALUE] = qlink(pidiv200);
+
+	/* return pi/200 */
+	return pidiv200;
+}
+
+
 /*
  * Calculate the exponential function to the nearest or next to nearest
  * multiple of the positive number epsilon.
